function parseSettingsFromJson(name) {
    let $block = $('#'+name);

    let settings = {};

    let settingsRaw = $.parseJSON($block.html() || '{}');
    $block.remove();
    return settingsRaw;
}

$(() => {
    var rates = parseSettingsFromJson("json_rates");
    var students = parseSettingsFromJson("json_students");
    var groups = parseSettingsFromJson("json_groups");
    var modules = parseSettingsFromJson("json_modules");
    var discipline = parseSettingsFromJson("json_discipline");
    console.log(discipline);
    var columnDescription = [
        {
            title: "ФИО",
            field: "fullName",
            frozen: true,
            width: 160
        },

    ];

    var studentList = Array();

    var fillFullNames = function (studentList) {
        studentList.forEach((student) => {
            student.fullName = [student.LastName, student.FirstName, student.SecondName].join(' ');
        })
    }

    let table = null;
//Create rate editor
    var rateEditor = function(cell, onRendered, success, cancel){
        //cell - the cell component for the editable cell
        //onRendered - function to call when the editor has been rendered
        //success - function to call to pass the successfuly updated value to Tabulator
        //cancel - function to call to abort the edit and return to a normal cell

        //create and style input
        var cellValue = cell.getValue(),
            input = document.createElement("input");
        let student = cell._cell.row.data;
        //TODO: no magic numbers, select text after "Submodule"
        let submoduleID = parseInt(cell._cell.column.field.substring(9,20));


        if (cellValue === undefined) {
            cellValue = 0;
        }

        input.setAttribute("type", "number");

        input.style.padding = "4px";
        input.style.width = "100%";
        input.style.boxSizing = "border-box";

        input.value = cellValue;

        onRendered(function(){
            input.focus();
            input.style.height = "100%";
        });

        function onChange(){
            success(input.value);
            console.log(student.RecordBookID, submoduleID, input.value);
            // TODO: recalculate totals
            student["Submodule"+submoduleID] = input.value;

            table.updateData([student]);
            //studentList.find((student) => {
            //    return student.RecordBookID === student.RecordBookID
            //})
            //cancel()
        }

        //submit new value on blur or change
        input.addEventListener("blur", onChange);

        //submit new value on enter
        input.addEventListener("keydown", function(e){
            if(e.keyCode == 13){
                onChange();
            }

            if(e.keyCode == 27){
                cancel();
            }
        });

        return input;
    };

    let extraModule = null;
    let extraSubmodule1 = null;
    let extraSubmodule2 = null;

    Object.keys(modules).forEach((moduleID) => {
        module = modules[moduleID];

        moduleDescription = {
            title: module.Name,
            columns: []
        };
        if (module.Type === "extra") {
            extraModule = module;
            extraSubmodule1 = extraModule.Submodules[0];
            extraSubmodule2 = extraModule.Submodules[1];
        } else {
            module.Submodules.forEach((submodule) => {
                moduleDescription.columns.push({
                    title: submodule.Name + " (" + submodule.Rate + ")",
                    field: "Submodule" + submodule.ID,
                    editor: rateEditor,
                    headerVertical: true,
                });
            });
        }
        columnDescription.push(moduleDescription);
    });

// TODO: bonus after total
    columnDescription.push({
        title: "Итог за семестр",
        field: "subtotal",
        headerVertical: false,

    });

    columnDescription.push({
        title: "Добор",
        field: "extra",
        headerVertical: false,

    });

    columnDescription.push({
        title: "Итог",
        field: "total",
        headerVertical: false,
    });

    Object.keys(students).forEach((groupID) => {
        fillFullNames(students[groupID]);
        students[groupID].forEach((student) => {
            student.groupID = groupID;
        });
        studentList.push(...students[groupID]);

    });

    studentList.forEach((student) => {
        recordBookID = student.RecordBookID;
        let extraRate = 0;
        let totalRate = 0;
        if (rates.hasOwnProperty(recordBookID)) {
            Object.keys(rates[recordBookID]).forEach((submoduleID) => {
                if (extraSubmodule1.ID === submoduleID || extraSubmodule2.ID === submoduleID) {
                    extraRate += rates[recordBookID][submoduleID];
                } else {
                    let propName = "Submodule" + submoduleID;
                    let rate = rates[recordBookID][submoduleID];
                    student[propName] = rate;
                    totalRate += rate;
                }
            });
        }
        if (extraRate > 0) {
            student.extra = extraRate;
        }
        student.subtotal = totalRate;
        student.total = totalRate + extraRate;
    });

    table = new Tabulator("#studentsRate", {
        columnVertAlign: "bottom", //align header contents to bottom of cell
        data: studentList,
        index: "RecordBookID",
        height: "800px",
        //layout: "fitColumns",
        persistentLayout: true,
        persistentSort: true,
        persistenceID: "Discipline"+discipline.ID,
        clipboard:true,
        history:true,
        historyUndo:function(action, component, data){
            console.log(data);
            console.log(component);
            console.log(action);
            //action - the action that has been undone
            //component - the Component object afected by the action (colud be a row or cell component)
            //data - the data being changed
        },
        historyRedo:function(action, component, data){
            console.log(data); // oldValue, newValue
            console.log(component);
            console.log(action); //cellEdit has action._cell.row/column
            //action - the action that has been redone
            //component - the Component object afected by the action (colud be a row or cell component)
            //data - the data being changed
        },
        clipboardPasteAction:"replace",
        columns: columnDescription,
        groupBy: "GroupID",
        groupHeader: function (value, count, data, group) {
            groupInfo = groups[value];
            return groupInfo.GroupNum + " группа " + groupInfo.GradeID + " курса";
        }
    });

//trigger download of data.xlsx file
    $("#download-xlsx").click(function () {
        table.download("xlsx", "data.xlsx", {sheetName: "My Data"});
    });

//trigger download of data.pdf file
    $("#download-pdf").click(function () {
        //table.download("pdf", "data.pdf", {
        //autoTable: function(doc) {
        // need to use https://github.com/sphilee/jsPDF-CustomFonts-support to generate base64-encoded string
//    doc.addFileToVFS("fontawesome-webfont.ttf", "AAEAAAAOAIAAAwBgRkZUTW..."); // your font in binary format as second parameter
        //doc.addFont("fontawesome-webfont.ttf", "FontAwesome", "normal");
        //doc.setFont("FontAwesome");
        //return {
//      styles: {
        //font: "FontAwesome",
        //fontStyle: "normal"
        //}
        //};
        //}
//});
        table.download("pdf", "data.pdf", {
            orientation: "portrait", //set page orientation to portrait
            title: "Example Report", //add title to report
        });
    });


//TODO: use history callbacks to track changes when using undo/redo
//http://tabulator.info/docs/4.2/callbacks#history

//undo button
    $("#history-undo").on("click", function(){
        table.undo();
    });

//redo button
    $("#history-redo").on("click", function(){
        table.redo();
    });

// example add column
//table.addColumn({title:"ID", field: "GroupID" ,width:100, align:"center"});


    /* var table = new Tabulator("#example-table", {
        columnVertAlign:"bottom", //align header contents to bottom of cell
        data: tabledata,
        columns:[
        {title:"Name", field:"name", width:160},
        {//create column group
            title:"Work Info",
            columns:[
            {title:"Progress", field:"progress", align:"right", sorter:"number", width:100},
            {title:"Rating", field:"rating", align:"center", width:80},
            {title:"Driver", field:"car", align:"center", width:80},
            ],
        },
        {//create column group
            title:"Personal Info",
            columns:[
            {title:"Gender", field:"gender", width:90},
            {title:"Favourite Color", field:"col", width:140},
            {title:"Date Of Birth", field:"dob", align:"center", sorter:"date", width:130},
            ],
        },
        ],
    }); */
});
